#! /usr/bin/env bash
set -euo pipefail

sudo snap install jq

echo "Stopping k3s service for backup"
/usr/local/bin/k3s-killall.sh
echo "Backing up all PVCs to /var/lib/OpenAppStack.tar.gz"
cd /var/lib
tar -czf OpenAppStack.tar.gz OpenAppStack
cd -
echo "Starting k3s service"
sudo systemctl start k3s

echo "Making new namespaces"

kubectl create ns stackspin
kubectl create ns stackspin-apps

echo "adding stackspin-named secrets"
kubectl get secret -n flux-system oas-cluster-variables -o json | jq '.metadata.name="stackspin-cluster-variables"' | kubectl apply -f -
kubectl get secret -n flux-system oas-single-sign-on-variables -o json | jq '.metadata.name="stackspin-single-sign-on-variables"' | kubectl apply -f -
kubectl get secret -n flux-system oas-oauth-variables -o json | jq '.metadata.name="stackspin-oauth-variables"' | kubectl apply -f -
kubectl get secret -n flux-system oas-kube-prometheus-stack-variables -o json | jq '.metadata.name="stackspin-kube-prometheus-stack-variables"' | kubectl apply -f -
kubectl get secret -n flux-system oas-nextcloud-variables -o json | jq '.metadata.name="stackspin-nextcloud-variables"' | kubectl apply -f -
kubectl get secret -n flux-system oas-wekan-variables -o json | jq '.metadata.name="stackspin-wekan-variables"' | kubectl apply -f -
kubectl get secret -n flux-system oas-rocketchat-variables -o json | jq '.metadata.name="stackspin-rocketchat-variables"' | kubectl apply -f -
kubectl get secret -n flux-system oas-wordpress-variables -o json | jq '.metadata.name="stackspin-wordpress-variables"' | kubectl apply -f -

# Backs up pvcs to /tmp/$namespace/$pvc.yaml and sets their pv's to "retain"
backup_pvc() {
  pvc=$1
  old_namespace=$2
  new_namespace=$3

  pv="$(kubectl get pvc -n "$old_namespace" "$pvc" --template '{{ .spec.volumeName }}')"
  pv_path="$(kubectl get pv "$pv" --template '{{ .spec.local.path }}')"

  echo "Backing up PVC '$pvc' with PV '$pv' from namespace '$old_namespace'"
  echo "PV path: '$pv_path'"

  # Make namespace backup directory
  mkdir -p "/tmp/$old_namespace"

  # back-up PVC and PV manifests
  kubectl get pvc -n "$old_namespace" "$pvc" -o yaml >"/tmp/$old_namespace/$pvc.yaml"
  kubectl get pv "$pv" -o yaml >"/tmp/$old_namespace/$pv.yaml"

  # Set reclaimpolicy for PV
  kubectl patch pv "$pv" -p '{"spec":{"persistentVolumeReclaimPolicy":"Retain"}}'

  # Re-add PVC in new namespace
  grep -v -e "uid:" -e "resourceVersion:" -e "namespace:" -e "selfLink:" "/tmp/$old_namespace/$pvc.yaml" | kubectl -n "$new_namespace" apply -f -

  # Get new pvc UID
  uid=$(kubectl get -n "$new_namespace" pvc "$pvc" --template '{{ .metadata.uid }}')

  # Set new PVC UID in PV
  kubectl patch pv "$pv" -p '{"spec":{"claimRef":{"uid":"'"$uid"'","name":null}}}'

  echo "When all is fine, re-set the reclaimPolicy to Delete:"
  echo "kubectl patch pv \"'$pv'\" -p '{\"spec\":{\"persistentVolumeReclaimPolicy\":\"Delete\"}}'"
}

# Move all PVCs to stackspin namespace
oas_pvcs="single-sign-on-userbackend storage-loki-0 grafana alertmanager-kube-prometheus-stack-alertmanager-db-alertmanager-kube-prometheus-stack-alertmanager-0 prometheus-kube-prometheus-stack-prometheus-db-prometheus-kube-prometheus-stack-prometheus-0"
for pvc in $oas_pvcs; do
  backup_pvc "$pvc" oas stackspin
done

# Move all PVCs to stackspin-apps namespace
oas_apps_pvcs="nextcloud-postgresql nextcloud-mariadb nextcloud-onlyoffice-data nextcloud-files wekan wordpress-files wordpress-mariadb"
for pvc in $oas_apps_pvcs; do
  backup_pvc "$pvc" oas-apps stackspin-apps
done

# Remove the old OAS kustomizations
flux delete --silent kustomization openappstack || echo "kustomization 'openappstack' not found"
flux delete --silent kustomization infrastructure || echo "kustomization 'infrastructure' not found"
flux delete --silent kustomization core || echo "kustomization 'core' not found"

# Remove old add-apps kustomizations
apps="monitoring  nextcloud  rocketchat  velero  wekan  wordpress"
for app in $apps; do
  flux delete --silent kustomization "add-$app" || echo "kustomization 'add-$app' not found"
  flux delete --silent kustomization "$app" || echo "kustomization '$app' not found"
done

# Remove old git repository
flux delete --silent source git openappstack

echo "Now check if all the Flux kustomizations and HelmReleases and the 'oas' and 'oas-apps' namespace have disappeared. If they haven't, wait until they have. Then you can re-run the install-stackspin.sh script and after that, re-run install-apps.sh <app> for each app you want to (re)install."

echo "Refer to the upgrade guide <https://docs.stackspin.net/en/latest/upgrading.html> for app specific upgrades."
